==> schemastery-gen-v1.0.1.zip: schemastery-gen-v1.0.1/src/decorators/register.ts <==
import {
  SchemaClassOptions,
  SchemaOptions,
  SchemaOptionsDict,
  TypeFromClass,
} from '../def';
import Schema from 'schemastery';
import { reflector } from '../metadata/reflector';
import _ from 'lodash';
import { Metadata } from '../metadata/metadata';

function getBasePropertySchemaFromOptions(options: SchemaOptions) {
  if (options.schema) {
    return options.schema;
  }
  if (typeof options.type !== 'string') {
    return schemaFromClass(options.type);
  }
  switch (options.type as string) {
    case 'any':
      return Schema.any();
    case 'never':
      return Schema.never();
    case 'string':
      return Schema.string();
    case 'number':
      return Schema.number();
    case 'boolean':
      return Schema.boolean();
    case 'object':
      return Schema.object({}, true);
    default:
      return Schema.any();
  }
}

function applyOptionsToSchema(schema: Schema, options: SchemaClassOptions) {
  if (options.required != undefined) {
    schema.meta.required = options.required;
  }
  if (options.hidden != undefined) {
    schema.meta.hidden = options.hidden;
  }
  if (options.default != undefined) {
    schema.meta.default = options.default;
  }
  if (options.comment != undefined) {
    schema.meta.comment = options.comment;
  }
  if (options.allowUnknown != undefined) {
    schema.flag = options.allowUnknown;
  }
  if (options.desc != undefined) {
    schema.desc = options.desc;
  }
}

function getPropertySchemaFromOptions<PT>(options: SchemaOptions): Schema<PT> {
  let schema = getBasePropertySchemaFromOptions(options);
  if (options.dict) {
    schema = Schema.dict(schema);
  }
  if (options.array) {
    schema = Schema.array(schema);
  }
  applyOptionsToSchema(schema, options);
  return schema;
}

function schemasFromDict<T>(dict: SchemaOptionsDict<T>): Schema<T> {
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore
  return Schema.object<T>(
    _.mapValues(dict, (opt) => getPropertySchemaFromOptions(opt)),
  );
}

function schemaOptionsFromClass<C extends { new (...args: any[]): any }>(
  cl: C,
): SchemaOptionsDict<TypeFromClass<C>> {
  const keys = reflector.getArray(
    'SchemaMetaKey',
    cl,
  ) as (keyof TypeFromClass<C> & (string | symbol))[];
  if (!keys) {
    return null;
  }
  const result: SchemaOptionsDict<TypeFromClass<C>> = {};
  for (const key of keys) {
    const option = reflector.get('SchemaMeta', cl, key);
    result[key] = option;
  }
  return result;
}

export function schemaFromClass<C extends { new (...args: any[]): any }>(
  cl: C,
): Schema<TypeFromClass<C>> {
  let schema: Schema;
  const optionsDict = schemaOptionsFromClass(cl);
  if (!optionsDict) {
    schema = Schema.object({}, true);
  } else {
    schema = schemasFromDict<TypeFromClass<C>>(optionsDict);
  }
  const classOptions = reflector.get('SchemaClassOptions', cl);
  if (classOptions) {
    applyOptionsToSchema(schema, classOptions);
  }
  return schema;
}

const schemaFields: (keyof Schema.Base)[] = [
  'type',
  'desc',
  'key',
  'flag',
  'value',
  'alt',
  'sDict',
  'list',
  'dict',
  'callback',
  'meta',
];

function applySchemaForClass<C extends { new (...args: any[]): any }>(
  originalClass: C,
  instance: TypeFromClass<C>,
  originalObject: any,
) {
  const schema = schemaFromClass(originalClass);
  const newRawObject = new schema(originalObject);
  for (const key in schema.dict) {
    const transformer = reflector.get('Transformer', originalClass, key);
    if (transformer) {
      newRawObject[key] = transformer(newRawObject[key]);
    }
  }
  for (const key in newRawObject) {
    Object.defineProperty(instance, key, {
      writable: true,
      enumerable: true,
      configurable: true,
      value: newRawObject[key],
    });
  }
  return instance;
}

export function RegisterSchema(options: SchemaClassOptions = {}) {
  return function <C extends { new (...args: any[]): any }>(originalClass: C) {
    Metadata.set('SchemaClassOptions', options)(originalClass);
    const schema = schemaFromClass(originalClass);
    const newClass = (function (...args: any[]): any {
      const instance = new originalClass(...args);
      const originalObject = args[0];
      return applySchemaForClass(originalClass, instance, originalObject);
    } as unknown) as C & Schema<TypeFromClass<C>>;
    Object.defineProperty(newClass, 'name', {
      value: originalClass.name,
    });
    Object.setPrototypeOf(newClass, originalClass.prototype);
    for (const field of schemaFields) {
      // eslint-disable-next-line @typescript-eslint/ban-ts-comment
      // @ts-ignore
      newClass[field] = schema[field];
    }
    return newClass;
  };
}

// for backward compatibility
export const SchemaConf = RegisterSchema;

==> schemastery-gen-v1.0.2.zip: schemastery-gen-v1.0.2/src/decorators/register.ts <==
import {
  ClassType,
  SchemaClassOptions,
  SchemaOptions,
  SchemaOptionsDict,
  TypeFromClass,
} from '../def';
import Schema from 'schemastery';
import { reflector } from '../metadata/reflector';
import _ from 'lodash';
import { Metadata } from '../metadata/metadata';

function getBasePropertySchemaFromOptions(options: SchemaOptions) {
  if (options.schema) {
    return options.schema;
  }
  if (typeof options.type !== 'string') {
    return schemaFromClass(options.type);
  }
  switch (options.type as string) {
    case 'any':
      return Schema.any();
    case 'never':
      return Schema.never();
    case 'string':
      return Schema.string();
    case 'number':
      return Schema.number();
    case 'boolean':
      return Schema.boolean();
    case 'object':
      return Schema.object({}, true);
    default:
      return Schema.any();
  }
}

function applyOptionsToSchema(schema: Schema, options: SchemaClassOptions) {
  if (options.required != undefined) {
    schema.meta.required = options.required;
  }
  if (options.hidden != undefined) {
    schema.meta.hidden = options.hidden;
  }
  if (options.default != undefined) {
    schema.meta.default = options.default;
  }
  if (options.comment != undefined) {
    schema.meta.comment = options.comment;
  }
  if (options.allowUnknown != undefined) {
    schema.flag = options.allowUnknown;
  }
  if (options.desc != undefined) {
    schema.desc = options.desc;
  }
}

function getPropertySchemaFromOptions<PT>(options: SchemaOptions): Schema<PT> {
  let schema = getBasePropertySchemaFromOptions(options);
  if (options.dict) {
    schema = Schema.dict(schema);
  }
  if (options.array) {
    schema = Schema.array(schema);
  }
  applyOptionsToSchema(schema, options);
  return schema;
}

function schemasFromDict<T>(dict: SchemaOptionsDict<T>): Schema<T> {
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-ignore
  return Schema.object<T>(
    _.mapValues(dict, (opt) => getPropertySchemaFromOptions(opt)),
  );
}

function schemaOptionsFromClass<C extends { new (...args: any[]): any }>(
  cl: C,
): SchemaOptionsDict<TypeFromClass<C>> {
  const keys = reflector.getArray(
    'SchemaMetaKey',
    cl,
  ) as (keyof TypeFromClass<C> & (string | symbol))[];
  if (!keys) {
    return null;
  }
  const result: SchemaOptionsDict<TypeFromClass<C>> = {};
  for (const key of keys) {
    const option = reflector.get('SchemaMeta', cl, key);
    result[key] = option;
  }
  return result;
}

export function schemaFromClass<T>(cl: ClassType<T>): Schema<T> {
  let schema: Schema;
  const optionsDict = schemaOptionsFromClass(cl);
  if (!optionsDict) {
    schema = Schema.object({}, true);
  } else {
    schema = schemasFromDict<T>(optionsDict);
  }
  const classOptions = reflector.get('SchemaClassOptions', cl);
  if (classOptions) {
    applyOptionsToSchema(schema, classOptions);
  }
  return schema;
}

const schemaFields: (keyof Schema.Base)[] = [
  'type',
  'desc',
  'key',
  'flag',
  'value',
  'alt',
  'sDict',
  'list',
  'dict',
  'callback',
  'meta',
];

function applySchemaForClass<T>(
  originalClass: ClassType<T>,
  instance: T,
  originalObject: any,
) {
  const schema = schemaFromClass(originalClass);
  const newRawObject = new schema(originalObject);
  for (const key in schema.dict) {
    const transformer = reflector.get('Transformer', originalClass, key);
    if (transformer) {
      newRawObject[key] = transformer(newRawObject[key]);
    }
  }
  for (const key in newRawObject) {
    Object.defineProperty(instance, key, {
      writable: true,
      enumerable: true,
      configurable: true,
      value: newRawObject[key],
    });
  }
  return instance;
}

export function RegisterSchema(options: SchemaClassOptions = {}) {
  return function <T>(originalClass: ClassType<T>) {
    Metadata.set('SchemaClassOptions', options)(originalClass);
    const schema = schemaFromClass(originalClass);
    const newClass = (function (...args: any[]): any {
      const instance = new originalClass(...args);
      const originalObject = args[0];
      return applySchemaForClass(originalClass, instance, originalObject);
    } as unknown) as ClassType<T> & Schema<T>;
    Object.defineProperty(newClass, 'name', {
      value: originalClass.name,
    });
    Object.setPrototypeOf(newClass, originalClass.prototype);
    for (const field of schemaFields) {
      // eslint-disable-next-line @typescript-eslint/ban-ts-comment
      // @ts-ignore
      newClass[field] = schema[field];
    }
    return newClass;
  };
}

// for backward compatibility
export const SchemaConf = RegisterSchema;
